@@ -927,7 +927,10 @@ void SignonSessionCore::destroy()
     else
         sessionsOfNonStoredCredentials.removeOne(this);
 
-    emit destroyed();
+    QObjectList authSessions;
+    while (authSessions = children(), !authSessions.isEmpty()) {
+        delete authSessions.first();
+    }
     deleteLater();
 }
 
